# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'miniScope.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(518, 354)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Form)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(1)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.splitter = QtWidgets.QSplitter(Form)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.frame_2 = QtWidgets.QFrame(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(90)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
        self.frame_2.setSizePolicy(sizePolicy)
        self.frame_2.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.frame_2)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.plot = PlotWidget(self.frame_2)
        self.plot.setObjectName("plot")
        self.verticalLayout.addWidget(self.plot)
        self.message = QtWidgets.QLabel(self.frame_2)
        self.message.setStyleSheet("color: rgb(239, 41, 41);\n"
"font: 57 12pt \"Ubuntu\";")
        self.message.setText("")
        self.message.setObjectName("message")
        self.verticalLayout.addWidget(self.message)
        self.frame = QtWidgets.QFrame(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setVerticalSpacing(1)
        self.gridLayout.setObjectName("gridLayout")
        self.A1Enabled = QtWidgets.QCheckBox(self.frame)
        self.A1Enabled.setText("")
        self.A1Enabled.setChecked(True)
        self.A1Enabled.setObjectName("A1Enabled")
        self.gridLayout.addWidget(self.A1Enabled, 2, 0, 1, 1)
        self.A1Box = QtWidgets.QComboBox(self.frame)
        self.A1Box.setObjectName("A1Box")
        self.gridLayout.addWidget(self.A1Box, 2, 1, 1, 2)
        self.label_2 = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 3)
        self.frame_3 = QtWidgets.QFrame(self.frame)
        self.frame_3.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_3.setObjectName("frame_3")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.frame_3)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setSpacing(2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.timebaseSlider = QtWidgets.QSlider(self.frame_3)
        self.timebaseSlider.setMaximum(7)
        self.timebaseSlider.setOrientation(QtCore.Qt.Horizontal)
        self.timebaseSlider.setTickPosition(QtWidgets.QSlider.TicksAbove)
        self.timebaseSlider.setObjectName("timebaseSlider")
        self.horizontalLayout_2.addWidget(self.timebaseSlider)
        self.timebaseLabel = QtWidgets.QLabel(self.frame_3)
        self.timebaseLabel.setObjectName("timebaseLabel")
        self.horizontalLayout_2.addWidget(self.timebaseLabel)
        self.gridLayout.addWidget(self.frame_3, 0, 0, 1, 3)
        self.gridLayout.setColumnStretch(2, 10)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.A2Box = QtWidgets.QCheckBox(self.frame)
        self.A2Box.setObjectName("A2Box")
        self.verticalLayout_2.addWidget(self.A2Box)
        self.label = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label)
        self.line = QtWidgets.QFrame(self.frame)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.resultsLayout = QtWidgets.QVBoxLayout()
        self.resultsLayout.setObjectName("resultsLayout")
        self.list = QtWidgets.QListWidget(self.frame)
        self.list.setObjectName("list")
        self.resultsLayout.addWidget(self.list)
        self.verticalLayout_2.addLayout(self.resultsLayout)
        self.freqCheckBox = QtWidgets.QCheckBox(self.frame)
        self.freqCheckBox.setObjectName("freqCheckBox")
        self.verticalLayout_2.addWidget(self.freqCheckBox)
        self.verticalLayout_2.setStretch(4, 10)
        self.horizontalLayout.addWidget(self.splitter)

        self.retranslateUi(Form)
        self.list.currentRowChanged['int'].connect(Form.changeParameter)
        self.A1Box.currentTextChanged['QString'].connect(Form.changeChannel)
        self.timebaseSlider.valueChanged['int'].connect(Form.timebaseChanged)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label_2.setText(_translate("Form", "Channels"))
        self.timebaseLabel.setText(_translate("Form", "1mS"))
        self.A2Box.setText(_translate("Form", "A2"))
        self.label.setText(_translate("Form", "Results"))
        self.freqCheckBox.setText(_translate("Form", "Cross Check Freq"))

from pyqtgraph import PlotWidget
